/****** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XpcomViewer code.
 * 
 * The Initial Developer of the Original Code is NeutronStar, s.r.o.
 * Portions created by NeutronStar, s.r.o. are Copyright (C) 2005 
 * NeutronStar, s.r.o.. All Rights Reserved.
 * 
 * Contributor(s):
 *  - Ondrej Donek, <ondrejd@gmail.com>, original author
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 ****** END LICENSE BLOCK *****/


/**
 * Class that serves all what is needed for working with bookmarks 
 * datasource and its file.
 */
var gBookmarksDataSource = 
{
    /** 
     * Used XPCOM comonents 
     */
   	nsIRDFResource 	: Components.interfaces.nsIRDFResource,
	nsIRDFNode 		: Components.interfaces.nsIRDFNode,
	nsIRDFLiteral 	: Components.interfaces.nsIRDFLiteral,
	nsIRDFDS		: Components.interfaces.nsIRDFDataSource,
	nsILocalFile	: Components.interfaces.nsILocalFile,
	inMemoryDS 		: "@mozilla.org/rdf/datasource;1?name=in-memory-datasource",
	
	rdfSrv 			: Components.classes["@mozilla.org/rdf/rdf-service;1"].
    				  getService (Components.interfaces.nsIRDFService),
	IOSrv			: Components.classes["@mozilla.org/network/io-service;1"].
					  getService (Components.interfaces.nsIIOService),

	/** 
	 * Properties describing our rdf datasource 
	 */
	_ds : null,
	_dsFile : null,
	// Here are some getters for variables we oftenly use when working 
	// with our rdf datasource.
	get _namespace () { return "http://www.mozilla.org/NC-xpcom#"; },
	get _uriBase () { return "rdf:xpcomviewer:bookmarks"; },
    // resources 
	get _resBase () { return this.rdfSrv.GetResource (this._uriBase); },
    // arcs
    get _arcLabel () { return this.rdfSrv.GetResource (this._namespace + "label"); },
    get _arcTarget () { return this.rdfSrv.GetResource (this._namespace + "target"); },
    // root container
    _rootCont : null,

    
    /** 
     * Array of pointers to UI elements which are used for displaying 
     * bookmarks datasource.
     */
    _elements : [],
    
    
    /**
     * This class needs stringbundles but I don't want to care about 
     * including .properties file to every xul file where I used this 
     * class. So I loading this file by JavaScript and if this failed 
     * is used default set of stringbundles.
     * 
     * Note: Not translate these defaults string bundles!!!
     */
    _strBundles : null,
    __BUNDLES_LOADED_ : false,
    __defaultStrBundles : {
        targetNameObtainingFailedPrompt : "Obtaining of label for new " + 
                                          "bookmark failed.\nPlease, " + 
                                          "enter name:"
    },
    
    
    /**
     * Initialize this class. As argument pass in array of pointers 
     * to UI elements which are used for displaying bookmarks datasource 
     * and which have to be refreshed whetever any change on this 
     * bookmark is performed.
     * 
     * @param   nsIArray
     */
    init : function (aElements)
    {
		// load .preperties file with string bundles
        try 
		{
			var localeSrv = Components.classes["@mozilla.org/intl/nslocaleservice;1"].
				getService(Components.interfaces.nsILocaleService);
			
			var appLocale =  localeSrv.GetApplicationLocale();
			this._strBundles = Components.classes["@mozilla.org/intl/stringbundle;1"].
				getService(Components.interfaces.nsIStringBundleService).
                CreateBundle ("chrome://xpcomviewer/locale/bookmarks-datasource.properties", 
                appLocale);
            
            this.__BUNDLES_LOADED_ = true;
		} 
		catch (e) {
            this.__BUNDLES_LOADED_ = false;
		}
        
        // initialize rest
        this._elements = aElements;
        this.initializeDataSourceFile ();
		
		if (this._dsFile.exists ())
        {
		    this.loadDataSourceFromFile ();
            // initialize root container
            this._rootCont = Components.classes["@mozilla.org/rdf/container;1"].
                    createInstance (Components.interfaces.nsIRDFContainer).
                    Init (this._ds, this._resBase);
        }
		else
		{
		    this.createBlankDataSource ();
		    this.saveDataSourceToFile ();
		}
        
		this.refreshElements ();
    }, // end gBookmarksDataSource.init (aElements)
    
    	
	
	/////////////////////////////////////////////
	/// Bookmarks add/remove methods
    
    
    /**
     * Add new bookmark into bookmarks container, save datasource 
     * and refresh allBookmarks-menu.
     * 
     * @param   nsIRDFResource  Resource to which we apppend bookmark
     */
    addBookmark : function (aTargetRes)
    {
        // 1. obtain name of item from given resource 
        var targetName = "";
        var target = gXpcomViewer.mDataSource.GetTarget (aTargetRes, 
            gXpcomViewer.arcLabel, true);
        
        if (target instanceof this.nsIRDFLiteral)
            targetName = target.Value;
        
        if (targetName == "")
        {
            var promptTitle = prompt ((this.__BUNDLES_LOADED_) ? 
                this._strBundles.getString ("targetNameObtainingFailedPrompt") 
                : this.__defaultStrBundles.targetNameObtainingFailedPrompt);
            
            targetName = prompt (promptTitle);
        }    
        
        if (targetName =="") 
            return;
        
        // 2. create new bookmark
        var newBookmark = this.rdfSrv.GetResource (this._uriBase + ":" + 
            Math.round (Math.random () * 64 * 32 * 8));
           
        this._ds.Assert (newBookmark, this._arcLabel, 
            this.rdfSrv.GetLiteral (targetName), true);
        this._ds.Assert (newBookmark, this._arcTarget, 
            this.rdfSrv.GetLiteral (aTargetRes.Value), true);
        
        // 4. append bookmark into bookmarks container
        this._rootCont.AppendElement (newBookmark);
        
        // 5. save datasource and refresh all elements 
        this.saveDataSourceToFile ();
        this.refreshElements ();
    }, // end gBookmarksDataSource.addBookmark (aTargetRes)
	
	
	/**
     * Remove bookmark
     */
    removeBookmark : function ()
    {
        
    }, // end gBookmarksDataSource.removeBookmark ()
    
    
	/////////////////////////////////////////////
	/// DataSource related methods
    
    
    /**
     * Initialize bookmarks datasource file
     */
    initializeDataSourceFile : function ()
    {
		this._dsFile = Components.classes["@mozilla.org/file/directory_service;1"].
			getService(Components.interfaces.nsIProperties).
			get("ProfD", Components.interfaces.nsIFile);
		
		this._dsFile.append ("extensions");
		this._dsFile.append ("{85CBA252-45E8-4e00-AF4D-E7EE9D462A9B}");
		this._dsFile.append ("bookmarks.rdf");
    }, // end gBookmarksDataSource.initializeDataSourceFile ()
    
    
    /**
     * Create blank bookmarks datasource 
     */
    createBlankDataSource : function ()
    {
        this._ds = Components.classes[this.inMemoryDS].createInstance (this.nsIRDFDS);
        
        // initialize root container
        this._rootCont = Components.classes["@mozilla.org/rdf/container-utils;1"].
            createInstance (Components.interfaces.nsIRDFContainerUtils).
            MakeSeq (this._ds, this._resBase);
    }, // end gBookmarksDataSource.createBlankDataSource ()
    
    
    /**
     * Load bookmarks datasource from file
     */
    loadDataSourceFromFile : function ()
    {
		var stream1 = Components.classes["@mozilla.org/network/file-input-stream;1"].
			createInstance (Components.interfaces.nsIFileInputStream);
		var stream2 = Components.classes["@mozilla.org/scriptableinputstream;1"].
			createInstance (Components.interfaces.nsIScriptableInputStream);
		var parser = Components.classes["@mozilla.org/rdf/xml-parser;1"].
			createInstance (Components.interfaces.nsIRDFXMLParser);
		
		var rootUri = this.IOSrv.newURI (this._uriBase, null, null);
		var content = new String ("");
		
		this._ds = Components.classes[this.inMemoryDS].
			createInstance (this.nsIRDFDS);
		
		try
		{
			stream1.init (this._dsFile, 1, 0, false);
			stream2.init (stream1);
			content += stream2.read (-1);
			parser.parseString (this._ds, rootUri, content);
			stream2.close ();
			stream1.close ();
		}
		catch (e)
		{
			Components.utils.reportError ("Error when reading datasource from file!\n" + e);
		}
    }, // end gBookmarksDataSource.loadDataSourceFromFile ()
    
    
    /**
     * Save bookmarks datasource to file
     */
    saveDataSourceToFile : function ()
    {
		var outputStream = 
		{
			data : "",
			close : function () {},
			flush : function () {},
			write : function (buffer, count) 
			{
				this.data += buffer;
				return count;
			},
			writeFrom : function (stream, count) {},
			isNonBlocking : false
		}
		
		var serializer = Components.classes["@mozilla.org/rdf/xml-serializer;1"].
			createInstance(Components.interfaces.nsIRDFXMLSerializer);
		var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].
			createInstance (Components.interfaces.nsIFileOutputStream);
		
		try
		{	
			stream.init (this._dsFile, 0x02 | 0x08 | 0x20, 0664, 0);
			serializer.init (this._ds);
			serializer.QueryInterface (Components.interfaces.nsIRDFXMLSource);
			serializer.Serialize (stream);
			stream.close ();
		}
		catch (e)
		{
			Components.utils.reportError (e);
		} 
    }, // end gBookmarksDataSource.saveDataSource ()
    
    
    /**
     * Refresh elements which are used for displaying datasources
     */
    refreshElements : function ()
    {
	    for (var elm in this._elements)
        {
            var element = this._elements[elm];
            
            this.removeElementDataSources (element);
            element.database.AddDataSource (this._ds);
            element.builder.rebuild ();
        }
    }, // end gBookmarksDataSource.init (aElements)
	
	
	/**
	 * Removes all attached rdf datasources of given element.
	 * 
	 * @param	object
	 */
	removeElementDataSources : function (aElement)
	{
        if (typeof(aElement.database) != "undefined")
        {
            var datasources = aElement.database.GetDataSources ();
            
            while (datasources.hasMoreElements ())
            {
                var ds = datasources.getNext ();
                try
                {
                    aElement.database.RemoveDataSource (ds);
                }
                catch (e)
                { }
            }
		}
    } // end gBookmarksDataSource.init (aElements)
    
    
} // end of gBookmarksDataSource
